@@ -1,24 +1,26 @@
 import copy
+from dataclasses import dataclass
 import json
 from collections import defaultdict
 from pathlib import Path
+import string
 from typing import Set, Dict, Any, List, Type, Optional
 
 import tensorflow as tf  # type: ignore
 from tensorflow.python.keras.utils.generic_utils import func_load  # type: ignore
 
-from leap_model_parser.contract.importmodelresponse import NodeResponse, WrapperData
+from leap_model_parser.contract.importmodelresponse import DatasetInfo, NodeResponse, WrapperData
 
 layer_attributes_adaptation = {'bias_initializer', 'kernel_initializer'}
 
-
 class KerasJsonModelImport:
-    def __init__(self, custom_layers: Optional[Dict[str, Type[tf.keras.layers.Layer]]] = None) -> None:
+    def __init__(self, custom_layers: Optional[Dict[str, Type[tf.keras.layers.Layer]]] = None, dataset_info: Optional[DatasetInfo] = None) -> None:
         self.id = 1
         self.nodes_cache: Dict[str, NodeResponse] = defaultdict()
         self.layer_name_to_layer: Dict[str, Dict] = defaultdict()
         self.visited_connections: Set[str] = set()
         self.ui_components: Dict[str, Dict] = defaultdict()
+        self.dataset_info = dataset_info
 
         if custom_layers is None:
             custom_layers = {}
@@ -125,6 +127,8 @@ class KerasJsonModelImport:
         node_input_layer = self.nodes_cache[node_input_layer_name]
         node_output_key = self.get_connection_key(input_layer, node_input_layer, 'outputs', input_index)
         node_input_key = self.get_connection_key(input_layer, node_output_layer, 'inputs', input_index)
+        if node_output_key is None or node_input_key is None:
+            return
 
         if node_output_key not in node_input_layer.outputs:
             node_input_layer.outputs[node_output_key] = {'connections': []}
@@ -145,7 +149,7 @@ class KerasJsonModelImport:
         if layer_metadata is None:
             if layer_json['class_name'] in self.custom_layers:
                 layer_metadata = {
-                    'type': 'UserCustomLayer',
+                    'type': 'CustomLayer',
                     'name': layer_json['class_name']
                 }
         return layer_metadata
@@ -238,14 +242,18 @@ class KerasJsonModelImport:
             "output_name": "Action"
         })
 
-    def get_connection_key(self, node_input_layer, node: NodeResponse, direction: str, index: int) -> str:
+    def get_connection_key(self, node_input_layer, node: NodeResponse, direction: str, index: int) -> Optional[str]:
         layer_metadata = self.ui_components.get(node.name)
-        if node.name == 'Representation Block' or layer_metadata is None:
+        if _is_represntaion_block_node(node) or layer_metadata is None:
             if direction == 'inputs':
                 connection_format = f"{node.id}-input"
             else:
                 connection_format = f"{node.id}-feature_map"
-        elif node.name == 'Dataset':
+        elif _is_dataset_node(node):
+            inputs = _get_dataset_inputs_from_dataset_node(node)
+            has_dataset_but_no_input_is_match = self.dataset_info is not None and not any(input_data['name'] == node_input_layer['name'] for input_data in inputs)
+            if has_dataset_but_no_input_is_match:
+                return None
             connection_format = f"{node.id}-{node_input_layer['name']}"
         else:
             try:
@@ -278,14 +286,26 @@ class KerasJsonModelImport:
             layer['config'] = gru_config
         return layer
 
-    def handle_input_layer(self, layer: Dict[str, Any]):
-        if 'Dataset' not in self.nodes_cache:
+    def get_and_add_dataset_node_if_not_existed(self) -> NodeResponse : 
+        datasetNode = self.nodes_cache.get('Dataset')
+        if datasetNode is None:
             dataset_copy = copy.deepcopy(DATASET_DATA)
-            self.nodes_cache['Dataset'] = NodeResponse(id=str(self.id), name='Dataset', data=dataset_copy)
+            datasetNode = NodeResponse(
+                id=str(self.id), name='Dataset', data=dataset_copy)
+            if self.dataset_info is not None:
+                datasetNode.data['datasetVersion'] = self.dataset_info.dataset_version
+                datasetNode.data['selected_dataset'] = self.dataset_info.name
+            self.nodes_cache['Dataset'] = datasetNode
             self.id += 1
+        return datasetNode
 
-        dataset = self.nodes_cache['Dataset']
-        inputs = dataset.data['datasetVersion']['metadata']['setup']['inputs']
+    def handle_input_layer(self, layer: Dict[str, Any]):
+        datasetNode = self.get_and_add_dataset_node_if_not_existed()
+        if self.dataset_info is None:
+            self.handle_input_layer_without_dataset(layer, datasetNode)
+
+    def handle_input_layer_without_dataset(self, layer: Dict[str, Any], datasetNode: NodeResponse):
+        inputs = datasetNode.data['datasetVersion']['metadata']['setup']['inputs']
 
         if not any(input_data['name'] == layer['name'] for input_data in inputs):
             shape = layer['config']['batch_input_shape']
@@ -299,6 +319,14 @@ def _add_origin_name_to_nodes(nodes: Dict[str, NodeResponse]):
             node.data["origin_name"] = node.data["name"]
             node.data.pop("name")
 
+def _is_dataset_node(node: NodeResponse) -> bool:
+    return node.name == 'Dataset'
+
+def _is_represntaion_block_node(node: NodeResponse) -> bool:
+    return node.name == 'Representation Block'
+
+def _get_dataset_inputs_from_dataset_node(node: NodeResponse):
+    return node.data['datasetVersion']['metadata']['setup']['inputs']
 
 DATASET_DATA = {'datasetVersion': {'metadata': {'setup': {'preprocess': {'training_length': 0, 'validation_length': 0},
                                                           'inputs': [], 'metadata': [], 'outputs': [],
